@@ -18,7 +18,6 @@
 */
 
 #include "notificationmessage_p.h"
-
 #include "imapparser_p.h"
 
 #include <QtCore/QDebug>
@@ -252,6 +251,11 @@ QString NotificationMessage::toString() const
 
   if ( parentCollection() >= 0 )
     rv += QString::fromLatin1( "collection %1 " ).arg( parentCollection() );
+  else {
+    rv += QLatin1String( "unspecified parent collection " );
+  }
+
+  rv += QString::fromLatin1( "mimetype %1 " ).arg( mimeType().isEmpty() ? QLatin1String( "unknown" ) : mimeType() );
 
   switch ( operation() ) {
     case Add:
@@ -285,7 +289,7 @@ QString NotificationMessage::toString() const
   }
 
   if ( parentDestCollection() >= 0 )
-    rv += QString::fromLatin1( "to collection %1" ).arg( parentDestCollection() );
+    rv += QString::fromLatin1( " to collection %1" ).arg( parentDestCollection() );
 
   return rv;
 }
