@@ -41,6 +41,10 @@
 #define AKONADI_DBUS_STORAGEJANITOR_PATH "/Janitor"
 
 // Commands
+#define AKONADI_CMD_CAPABILITY "CAPABILITY"
+#define AKONADI_CMD_LOGIN "LOGIN"
+#define AKONADI_CMD_LOGOUT "LOGOUT"
+
 #define AKONADI_CMD_COLLECTIONCOPY "COLCOPY"
 #define AKONADI_CMD_COLLECTIONCREATE "CREATE"
 #define AKONADI_CMD_COLLECTIONDELETE "DELETE"
@@ -65,6 +69,7 @@
 #define AKONADI_PARAM_FULLPAYLOAD "FULLPAYLOAD"
 #define AKONADI_PARAM_ALLATTRIBUTES "ALLATTR"
 #define AKONADI_PARAM_CACHEONLY "CACHEONLY"
+#define AKONADI_PARAM_CHECKCACHEDPARTSONLY "CHECKCACHEDPARTSONLY"
 #define AKONADI_PARAM_EXTERNALPAYLOAD "EXTERNALPAYLOAD"
 #define AKONADI_PARAM_REVISION "REV"
 #define AKONADI_PARAM_SIZE "SIZE"
@@ -84,4 +89,6 @@
 #define AKONADI_PARAM_PERSISTENTSEARCH_QUERYLANG "QUERYLANGUAGE"
 #define AKONADI_PARAM_PERSISTENTSEARCH_QUERYSTRING "QUERYSTRING"
 
+#define AKONADI_PARAM_CAPABILITY_NOTIFY "NOTIFY"
+
 #endif
