@@ -46,11 +46,10 @@ class FetchHelper : public QObject
     bool parseStream( const QByteArray &responseIdentifier );
 
   Q_SIGNALS:
-    void responseAvailable( const Response& );
+    void responseAvailable( const Akonadi::Response& );
 
   private:
     void init();
-    void parseCommand( const QByteArray &line );
     void updateItemAccessTime();
     void triggerOnDemandFetch();
     QSqlQuery buildItemQuery();
@@ -58,6 +57,7 @@ class FetchHelper : public QObject
     QSqlQuery buildFlagQuery();
     void parseCommandStream();
     QStack<Collection> ancestorsForItem( Collection::Id parentColId );
+    static bool needsAccessTimeUpdate(const QStringList &parts);
 
   private:
     ImapStreamParser *mStreamParser;
@@ -68,6 +68,7 @@ class FetchHelper : public QObject
     QHash<Collection::Id, QStack<Collection> > mAncestorCache;
     int mAncestorDepth;
     bool mCacheOnly;
+    bool mCheckCachedPayloadPartsOnly;
     bool mFullPayload;
     bool mAllAttrs;
     bool mSizeRequested;
