@@ -24,35 +24,73 @@
 #include <QSqlQuery>
 #include <QThreadStorage>
 #include <QtCore/QHash>
+#include <QtCore/QTimer>
 
 using namespace Akonadi;
 
-static QThreadStorage< QHash<QString, QSqlQuery>* > g_queryCache;
+enum {
+  // After these seconds without activity the cache is cleaned
+  CLEANUP_TIMEOUT = 30 // seconds
+};
 
-static QHash<QString, QSqlQuery>* cache()
+class Cache : public QObject
+{
+  Q_OBJECT
+public:
+
+  Cache()
+  {
+    connect( &m_cleanupTimer, SIGNAL(timeout()), SLOT(cleanup()));
+    m_cleanupTimer.setSingleShot(true);
+  }
+
+  QSqlQuery query( const QString &queryStatement)
+  {
+    m_cleanupTimer.start( CLEANUP_TIMEOUT*1000 );
+    return m_cache.value( queryStatement );
+  }
+
+public Q_SLOTS:
+  void cleanup()
+  {
+    m_cache.clear();
+  }
+
+public: // public, this is just a helper class
+  QHash<QString, QSqlQuery> m_cache;
+  QTimer m_cleanupTimer;
+};
+
+static QThreadStorage<Cache*> g_queryCache;
+
+static Cache* perThreadCache()
 {
   if ( !g_queryCache.hasLocalData() )
-    g_queryCache.setLocalData( new QHash<QString, QSqlQuery>() );
+    g_queryCache.setLocalData( new Cache() );
+
   return g_queryCache.localData();
 }
 
-bool QueryCache::contains(const QString& queryStatement)
+bool QueryCache::contains(const QString &queryStatement)
 {
   if ( DbType::type( DataStore::self()->database() ) == DbType::Sqlite ) {
     return false;
   } else {
-    return cache()->contains( queryStatement );
+    return perThreadCache()->m_cache.contains( queryStatement );
   }
 }
 
-QSqlQuery QueryCache::query(const QString& queryStatement)
+QSqlQuery QueryCache::query(const QString &queryStatement)
 {
-  return cache()->value( queryStatement );
+  return perThreadCache()->query( queryStatement );
 }
 
-void QueryCache::insert(const QString& queryStatement, const QSqlQuery& query)
+void QueryCache::insert(const QString &queryStatement, const QSqlQuery &query)
 {
   if ( DbType::type( DataStore::self()->database() ) != DbType::Sqlite ) {
-    cache()->insert( queryStatement, query );
+    perThreadCache()->m_cache.insert( queryStatement, query );
   }
 }
+
+
+#include <querycache.moc>
